#include "qmllistwrappermodel.h"
#include <QDebug>

QmlListWrapperModel::QmlListWrapperModel(QObject *parent)
    : QAbstractListModel(parent)
{
    m_dataSource.append("Lorem");
    m_dataSource.append("ipsum");
    m_dataSource.append("dolor");
    m_dataSource.append("sit");
    m_dataSource.append("amet");
    m_dataSource.append("consectetur");
    m_dataSource.append("adipiscing");
    m_dataSource.append("elit");
    m_dataSource.append("Vivamus");
    m_dataSource.append("vulputate");
    m_dataSource.append("enim");
    m_dataSource.append("non");
    for (int i = 0 ; i < m_dataSource.length() ; ++i) {
        m_sortedDataIndexes.append(i);
    }
}

void QmlListWrapperModel::performRandomMove()
{
    const int from = qrand() % m_sortedDataIndexes.length();
    int to;
    do {
        to = qrand() % m_sortedDataIndexes.length();
    } while (from == to);
    const int toRow
            = to
            + (from < to ? 1 : 0);// when moving forward, model's destination index must be greater by 1 than move() argument
    beginMoveRows(QModelIndex(), from, from, QModelIndex(), toRow);
    m_sortedDataIndexes.move(from, to);
    endMoveRows();
}

void QmlListWrapperModel::randomize()
{
    QVector<int> newIndexes;
    for (int i = 0 ; i < m_sortedDataIndexes.length() ; ++i)
    {
        if (qrand() % 2)
            newIndexes.append(m_sortedDataIndexes.at(i));
        else
            newIndexes.prepend(m_sortedDataIndexes.at(i));
    }
    beginResetModel();
    m_sortedDataIndexes = newIndexes;
    endResetModel();
}

int QmlListWrapperModel::rowCount(const QModelIndex &parent) const
{
    if (parent.isValid()) {
        qWarning() << "Invalid parent!";
        return -1;
    }
    return m_dataSource.length();
}

QVariant QmlListWrapperModel::data(const QModelIndex &index, int role) const
{
    if (ObjectRole == role &&
        index.isValid() &&
        index.row() < m_sortedDataIndexes.length()) {
        int idx = m_sortedDataIndexes.at(index.row());
        if (idx < m_dataSource.length())
            return m_dataSource[idx];
    }
    qWarning() << "Data asked for role" << role << "(ObjectRole ==" << ObjectRole << "), returning empty QVariant";
    return QVariant();
}

QHash< int, QByteArray > QmlListWrapperModel::roleNames() const
{
    QHash< int, QByteArray > ret;
    ret[ObjectRole] = "internalName";
    return ret;
}

int QmlListWrapperModel::count() const
{
    return rowCount(QModelIndex());
}

int QmlListWrapperModel::itemOrder(int indexToUse) const
{
    if (indexToUse < m_sortedDataIndexes.length() &&
        indexToUse >= 0)
        return m_sortedDataIndexes.at(indexToUse);
    else
        return -1;
}
